﻿/*
 * Copyright (C) 2010 Guillaume Duc <guillaume.duc@telecom-paristech.fr>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;

namespace AttackWrapper
{
    class MatlabWrapper
    {
        private TraceDirectory traceDirectory;
        private ResultFile resultFile;

        private byte[] correctSubkey;

        private string traceDirectoryname;
        private string resultFilename;
        private int numKeyToAttack;
        private int numTraces;
        private int subkeyNum;

        private int currentTraceNum;
        private Trace currentTrace;
        private DateTime startTime;

        public MatlabWrapper(int numKeyToAttack, int numTraces, string resultFilename, string traceDirectoryname, int subkeyNum)
        {
            this.resultFile = new ResultFile();
            this.correctSubkey = new byte[Constants.TRACE_KEY_SIZE_BYTES];
            this.numKeyToAttack = numKeyToAttack;
            this.numTraces = numTraces;
            this.resultFilename = resultFilename;
            this.traceDirectoryname = traceDirectoryname;
            this.currentTraceNum = 0;
            this.currentTrace = null;
            this.subkeyNum = subkeyNum;
        }

        public void init()
        {
            traceDirectory = new TraceDirectory(traceDirectoryname, numKeyToAttack);
            traceDirectory.parseDirectory();
            traceDirectory.findKey();

            AES.AES_key_schedule(traceDirectory.attackedKeyBytes, correctSubkey, subkeyNum);

            resultFile.subkeyNum = (uint)subkeyNum;
            resultFile.numIterations = (uint)numTraces;
            resultFile.numKey = (uint)numKeyToAttack;
            Array.Copy(traceDirectory.attackedKeyBytes, resultFile.correctKey, Constants.TRACE_KEY_SIZE_BYTES);
            Array.Copy(correctSubkey, resultFile.correctSubkey, Constants.TRACE_KEY_SIZE_BYTES);

            resultFile.openToWrite(resultFilename);
            resultFile.writeHeaders();
        }

        public Trace getTrace()
        {
            currentTrace = traceDirectory.getTrace(currentTraceNum);
            startTime = DateTime.Now;
            return currentTrace;
        }

        public void saveResult(PartialResult partialResult)
        {
            DateTime endTime = DateTime.Now;
            TimeSpan duration = endTime - startTime;

            resultFile.writePartialResults(currentTrace, partialResult, duration.TotalSeconds);
            currentTraceNum++;
        }

        public void endAttack()
        {
            resultFile.close();
        }
    }
}
