﻿/*
 * Copyright (C) 2010 Guillaume Duc <guillaume.duc@telecom-paristech.fr>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using System.IO;
using System;

namespace AttackWrapper
{
    public class ResultFile
    {
        public uint formatVersion;
        public uint subkeyNum;
        public uint numIterations;
        public uint numKey;
        public byte[] correctKey;
        public byte[] correctSubkey;

        private BinaryWriter binaryWriter;
        private BinaryReader binaryReader;

        public ResultFile()
        {
            correctKey = new byte[Constants.TRACE_KEY_SIZE_BYTES];
            correctSubkey = new byte[Constants.TRACE_KEY_SIZE_BYTES];
            binaryWriter = null;
            binaryReader = null;
            formatVersion = Constants.RESULTS_BINARY_FORMAT_VERSION;
        }

        public void openToWrite(string resultFilename)
        {
            binaryWriter = new BinaryWriter(File.Open(resultFilename, FileMode.Create, FileAccess.Write));
        }

        public void openToRead(string resultFilename)
        {
            binaryReader = new BinaryReader(File.Open(resultFilename, FileMode.Open, FileAccess.Read));
        }

        public void writeHeaders()
        {
            binaryWriter.Write(formatVersion);
            binaryWriter.Write(subkeyNum);
            binaryWriter.Write(numIterations);
            binaryWriter.Write(numKey);
            binaryWriter.Write(correctKey);
            binaryWriter.Write(correctSubkey);
        }

        public void readHeaders()
        {
            formatVersion = binaryReader.ReadUInt32();
            subkeyNum = binaryReader.ReadUInt32();
            numIterations = binaryReader.ReadUInt32();
            numKey = binaryReader.ReadUInt32();
            correctKey = binaryReader.ReadBytes(Constants.TRACE_KEY_SIZE_BYTES);
            correctSubkey = binaryReader.ReadBytes(Constants.TRACE_KEY_SIZE_BYTES);
        }

        public void writePartialResults(Trace trace, PartialResult results, double timeConsumed)
        {
            binaryWriter.Write(trace.plaintext);
            binaryWriter.Write(trace.ciphertext);
            binaryWriter.Write(timeConsumed);
            binaryWriter.Write(results.subkey_num);

            for (int subkeyByte = 0; subkeyByte < 16; subkeyByte++)
            {
                for (int subkeyByteValue = 0; subkeyByteValue < 256; subkeyByteValue++)
                {
                    binaryWriter.Write(results.bytes[subkeyByte,subkeyByteValue]);
                }
            }

            binaryWriter.Flush();
        }

        public PartialResult readPartialResults()
        {
            double timeConsumed;
            byte[] plaintext;
            byte[] ciphertext;
            byte subkey;

            plaintext = binaryReader.ReadBytes(Constants.TRACE_PLAINTEXT_SIZE_BYTES);
            ciphertext = binaryReader.ReadBytes(Constants.TRACE_CIPHERTEXT_SIZE_BYTES);
            timeConsumed = binaryReader.ReadDouble();
            subkey = binaryReader.ReadByte();

            PartialResult partialResults = new PartialResult(subkey);

            for (int subkeyByte = 0; subkeyByte < 16; subkeyByte++)
            {
                for (int subkeyByteValue = 0; subkeyByteValue < 256; subkeyByteValue++)
                {
                    partialResults.bytes[subkeyByte, subkeyByteValue] = binaryReader.ReadByte();
                }
            }

            return partialResults;
        }

        public void close()
        {
            if (binaryWriter != null)
            {
                binaryWriter.Flush();
                binaryWriter.Close();
            }

            if (binaryReader != null)
                binaryReader.Close();
        }
    }
}
