﻿/*
 * Copyright (C) 2010 Guillaume Duc <guillaume.duc@telecom-paristech.fr>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.IO;

namespace AttackWrapper
{
    public class Trace
    {
        // The plaintext (16 unsigned bytes)
        public byte[] plaintext;

        // The ciphertext (16 unsigned bytes)
        public byte[] ciphertext;

        // The 3253 samples of the trace (16 bits signed value)
        public short[] samples;


        public Trace()
        {
            plaintext = new byte[Constants.TRACE_PLAINTEXT_SIZE_BYTES];
            ciphertext = new byte[Constants.TRACE_CIPHERTEXT_SIZE_BYTES];
            samples = new short[Constants.TRACE_NUM_POINTS];
        }

        public static Trace loadFromFile(String filename,
            byte[] plaintext, byte[] ciphertext)
        {
            StreamReader sr = new StreamReader(filename, System.Text.Encoding.ASCII);
            String line;
            Trace trace = new Trace();
            int nb_values = 0;
            short value;

            while ((line = sr.ReadLine()) != null)
            {
                if (line.StartsWith("#"))
                    continue;

                try
                {
                    value = Int16.Parse(line);
                    trace.samples[nb_values] = value;
                    nb_values++;

                    //Console.WriteLine("{0}", value); // DEBUG
                }
                catch (FormatException)
                {
                    continue;
                }
            }

            //Console.WriteLine("Total samples: {0}", nb_values); // DEBUG

            sr.Close();

            Array.Copy(plaintext, trace.plaintext, trace.plaintext.Length);
            Array.Copy(ciphertext, trace.ciphertext, trace.ciphertext.Length);

            return trace;
        }
    }
}