﻿/*
 * Copyright (C) 2010 Guillaume Duc <guillaume.duc@telecom-paristech.fr>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Collections.Generic;
using System.IO;

namespace AttackWrapper
{
    class TraceDirectory
    {
        private int numAttackedKey;
        private string attackedKeyHexa;
        public byte[] attackedKeyBytes;

        private int traceIndex;
        private string directoryName;
        private SortedList<TraceFileKey, TraceFile> traceList;

        public TraceDirectory(string directoryName, int numAttackedKey)
        {
            this.numAttackedKey = numAttackedKey;
            this.directoryName = directoryName;
            this.traceList = new SortedList<TraceFileKey, TraceFile>();
            this.traceIndex = -1;
            this.attackedKeyBytes = new byte[Constants.TRACE_KEY_SIZE_BYTES];
        }

        public void parseDirectory()
        {
            Console.WriteLine("Listing directory...");
            string[] files = Directory.GetFiles(directoryName, "*");

            Console.WriteLine("Building list of traces...");
            foreach (string file in files)
            {
                int k = file.IndexOf("k=");
                int p = file.IndexOf("m=");
                int c = file.IndexOf("c=");

                if ((k == -1) || (p == -1) || (c == -1))
                    continue;

                TraceFile traceFile = new TraceFile();
                TraceFileKey traceFileKey = new TraceFileKey();

                traceFileKey.ks = file.Substring(k + 2, 32);
                traceFileKey.ps = file.Substring(p + 2, 32);
                traceFile.ks = file.Substring(k + 2, 32);
                traceFile.ps = file.Substring(p + 2, 32);
                traceFile.cs = file.Substring(c + 2, 32);
                traceFile.filename = file;
                traceFile.update();

                traceList.Add(traceFileKey, traceFile);
                //Console.WriteLine(ks + " " + ps + " " + cs + " " + file);
            }

            Console.WriteLine("{0} traces read", traceList.Count);
        }

        public void findKey()
        {
            int k = -1;
            string ks = "";

            traceIndex = -1;

            do
            {
                traceIndex++;

                TraceFile tf = traceList.Values[traceIndex];
                attackedKeyHexa = tf.ks;
                if (attackedKeyHexa.CompareTo(ks) != 0)
                {
                    ks = attackedKeyHexa;
                    k++;
                }
            } while (k != numAttackedKey);

            Tools.hexaStringToByteArray(attackedKeyHexa, attackedKeyBytes);
        }

        public Trace getTrace(int traceNum)
        {
            TraceFile traceFile = traceList.Values[traceIndex + traceNum];

            if (traceFile.ks.CompareTo(attackedKeyHexa) != 0)
                throw new Exception("No more trace for the attacked key...");

            Trace trace = Trace.loadFromFile(traceFile.filename, traceFile.plaintext,
                traceFile.ciphertext);

            return trace;
        }


        class TraceFileKey : IComparable
        {
            internal string ks, ps;

            public int CompareTo(object obj)
            {
                if (obj is TraceFileKey)
                {
                    TraceFileKey tf = (TraceFileKey)obj;

                    int res = this.ks.CompareTo(tf.ks);

                    if (res != 0)
                        return res;

                    return this.ps.CompareTo(tf.ps);
                }

                throw new ArgumentException();
            }
        }

        class TraceFile
        {
            internal string ks, ps, cs;
            internal byte[] key, plaintext, ciphertext;
            internal string filename;

            public TraceFile()
            {
                key = new byte[Constants.TRACE_KEY_SIZE_BYTES];
                plaintext = new byte[Constants.TRACE_PLAINTEXT_SIZE_BYTES];
                ciphertext = new byte[Constants.TRACE_CIPHERTEXT_SIZE_BYTES];
            }

            public void update()
            {
                Tools.hexaStringToByteArray(ks, key);
                Tools.hexaStringToByteArray(ps, plaintext);
                Tools.hexaStringToByteArray(cs, ciphertext);
            }
        }
    }
}
